//------------------------------------------------------------------------------
//
// This software is provided "AS IS".  360Commerce MAKES NO
// REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE SOFTWARE,
// EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NON-INFRINGEMENT. 360Commerce shall not be liable for
// any damages suffered as a result of using, modifying or distributing this
// software or its derivatives. Permission to use, copy, modify, and distribute
// the software and its documentation for any purpose is hereby granted.
//
// ScannerPanel.java - The Scanner panel for POStest
//
//------------------------------------------------------------------------------
// contribution of interface and implementation Rory K. Shaw/Raleigh/IBM 6-28-04
//------------------------------------------------------------------------------
// final framework completed 7-15-2004 (oce, clearData)
//------------------------------------------------------------------------------
package com.jpos.POStest;


import javax.swing.*;

import java.awt.*;
import java.awt.event.*;

import jpos.*;
import jpos.events.*;

public class ScannerPanel extends Component implements DataListener, 
													   ErrorListener, 
													   ActionListener {

    protected MainButtonPanel mainButtonPanel;

    private Scanner scanner = null;
	private Scanner scanner2 = null;
	private boolean scn2Open = false;
	private int status = 0;

    public String defaultLogicalName = "SymbolScannerUSB";


    byte[] scanData = new byte[] {};
    byte[] scanDataLabel = new byte[] {};
    int    scanDataType;
	int    scanDataCount;
	boolean autoDisable;
    boolean dataEventEnabled;
    boolean deviceEnabled;
    boolean freezeEvents;
    boolean decodeData;
	boolean checkHealth;

    boolean updateDevice = true;


    private JTextField scanDataTextField;
    private JTextField scanDataLabelTextField;
    private JTextField scanDataTypeTextField;
	private JTextField scanDataCountField;
	private JTextField scanHealthField;

    private JCheckBox claimedCB;
    private JCheckBox autoDisableCB;
    private JCheckBox deviceEnabledCB;
    private JCheckBox autoDataEventEnableCB;
    private JCheckBox dataEventEnabledCB;
    private JCheckBox freezeEventsCB;
    private JCheckBox decodeDataCB;
	private JCheckBox checkHealthCB;
    
    private JButton clearDataButton;

    Runnable doUpdateGUI;

    public ScannerPanel() {}



    public Component make() {
        
        if(scanner == null)
        {
            scanner = new Scanner();
            scanner.addErrorListener(this);
            scanner.addDataListener(this);
            scanner2 = new Scanner();
            scanner2.addErrorListener(this);
            scanner2.addDataListener(this);
        }

        JPanel mainPanel = new JPanel(false);
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));

        
        mainButtonPanel = new MainButtonPanel(this, defaultLogicalName);
        mainPanel.add(mainButtonPanel);
        
        JPanel subPanel = new JPanel();
        subPanel.setLayout(new BoxLayout(subPanel, BoxLayout.X_AXIS));

        JPanel propPanel = new JPanel();
        propPanel.setLayout(new BoxLayout(propPanel, BoxLayout.Y_AXIS));
        claimedCB = new JCheckBox("Claimed");
        Font f = claimedCB.getFont();
        Font newf = new Font(f.getName(),Font.PLAIN,f.getSize());
        claimedCB.setFont(newf);
        propPanel.add(claimedCB);
        autoDisableCB = new JCheckBox("Auto disable");
        autoDisableCB.setFont(newf);
        propPanel.add(autoDisableCB);
        deviceEnabledCB = new JCheckBox("Device enabled");
        deviceEnabledCB.setFont(newf);
        propPanel.add(deviceEnabledCB);
		autoDataEventEnableCB = new JCheckBox("Auto data event enable");
		autoDataEventEnableCB.setFont(newf);
		propPanel.add(autoDataEventEnableCB);
        dataEventEnabledCB = new JCheckBox("Data event enabled");
        dataEventEnabledCB.setFont(newf);
        propPanel.add(dataEventEnabledCB);
        freezeEventsCB = new JCheckBox("Freeze events");
        freezeEventsCB.setFont(newf);
        propPanel.add(freezeEventsCB);
        decodeDataCB = new JCheckBox("Decode data");
        decodeDataCB.setFont(newf);
        propPanel.add(decodeDataCB);
		checkHealthCB = new JCheckBox("Check Health");
		checkHealthCB.setFont(newf);
		propPanel.add(checkHealthCB);
        propPanel.add(Box.createVerticalGlue());
        subPanel.add(propPanel);

		claimedCB.setEnabled(false);
        autoDisableCB.setEnabled(true);
        deviceEnabledCB.setEnabled(true);
		autoDataEventEnableCB.setEnabled(true);
        dataEventEnabledCB.setEnabled(true);
        freezeEventsCB.setEnabled(true);
        decodeDataCB.setEnabled(true);
		checkHealthCB.setEnabled(true);

        CheckBoxListener cbListener = new CheckBoxListener();
        autoDisableCB.addItemListener(cbListener);
        deviceEnabledCB.addItemListener(cbListener);
        dataEventEnabledCB.addItemListener(cbListener);
        freezeEventsCB.addItemListener(cbListener);
        decodeDataCB.addItemListener(cbListener);
		checkHealthCB.addItemListener(cbListener);

        JPanel labelPanel = new JPanel();
        labelPanel.setLayout(new BoxLayout(labelPanel, BoxLayout.Y_AXIS));

        JLabel scanDataLabel = new JLabel("Scan Data: ");
        scanDataLabel.setMaximumSize(new Dimension(Short.MAX_VALUE, 25));
        labelPanel.add(scanDataLabel);
        labelPanel.add(Box.createRigidArea(new Dimension(0,4)));
        scanDataLabel = new JLabel("Scan Data Type: ");
        scanDataLabel.setMaximumSize(new Dimension(Short.MAX_VALUE, 25));
        labelPanel.add(scanDataLabel);
        labelPanel.add(Box.createRigidArea(new Dimension(0,4)));
        scanDataLabel = new JLabel("Scan Data Label: ");
        scanDataLabel.setMaximumSize(new Dimension(Short.MAX_VALUE, 25));
        labelPanel.add(scanDataLabel);
		labelPanel.add(Box.createRigidArea(new Dimension(0, 4)));
		scanDataLabel = new JLabel("Scan Data Count: ");
		scanDataLabel.setMaximumSize(new Dimension(Short.MAX_VALUE, 25));
		labelPanel.add(scanDataLabel);
		labelPanel.add(Box.createRigidArea(new Dimension(0, 4)));
		scanDataLabel = new JLabel("Scanner Health: ");
		scanDataLabel.setMaximumSize(new Dimension(Short.MAX_VALUE, 25));
		labelPanel.add(scanDataLabel);
		labelPanel.add(Box.createVerticalGlue());


        subPanel.add(labelPanel);

        JPanel tfPanel = new JPanel();
        tfPanel.setLayout(new BoxLayout(tfPanel, BoxLayout.Y_AXIS));

        scanDataTextField = new JTextField(30);
        scanDataTextField.setMaximumSize(new Dimension(Short.MAX_VALUE, 25));
        tfPanel.add(scanDataTextField);
        scanDataTypeTextField = new JTextField(30);
        scanDataTypeTextField.setMaximumSize(new Dimension(Short.MAX_VALUE, 25));
        tfPanel.add(scanDataTypeTextField);
        scanDataLabelTextField = new JTextField(30);
        scanDataLabelTextField.setMaximumSize(new Dimension(Short.MAX_VALUE, 25));
        tfPanel.add(scanDataLabelTextField);
		scanDataCountField = new JTextField(30);
		scanDataCountField.setMaximumSize(new Dimension(Short.MAX_VALUE, 25));
		tfPanel.add(scanDataCountField);
		scanHealthField = new JTextField(30);
		scanHealthField.setMaximumSize(new Dimension(Short.MAX_VALUE, 25));
		tfPanel.add(scanHealthField);
        
        clearDataButton = new JButton("Clear Data");
        tfPanel.add(clearDataButton);

        tfPanel.add(Box.createVerticalGlue());

        subPanel.add(tfPanel);

        mainPanel.add(subPanel);
        mainPanel.add(Box.createVerticalGlue());
  
        doUpdateGUI = new Runnable() {
          public void run() {
              updateGUI();
          }
        };

        clearDataButton.addActionListener(new ActionListener()
        {
        	public void actionPerformed(ActionEvent event)
        	{
        		scanData = new byte[] {};
        	    scanDataType = 0;
        		scanDataTextField.setText(new String(scanData));
        	    scanDataLabelTextField.setText("");
        	    scanDataTypeTextField.setText(Integer.toString(scanDataType));
                try
				{
					if (scanner.getClaimed())
						scanDataCount = scanner.getDataCount();
					else
						scanDataCount = 0;
				}
				catch(JposException e)
				{
				}
				scanDataCountField.setText(Integer.toString(scanDataCount));
        	}
        });

        return mainPanel;
    }

        public void actionPerformed(ActionEvent ae) {
             mainButtonPanel.action(ae);
             String logicalName = mainButtonPanel.getLogicalName();
             if(ae.getActionCommand().equals("open")){
                 try{
                     if(logicalName.equals("")){
                         logicalName = defaultLogicalName;
                     }

                     scanner.open(logicalName);
                 }catch(JposException e){
                     JOptionPane.showMessageDialog(null, "Failed to open \""+logicalName+"\"\nException: "+ e.getMessage(), "Failed", JOptionPane.ERROR_MESSAGE);
                     System.err.println("Jpos exception on open " + e);
                 }
             }
             else if(ae.getActionCommand().equals("claim")){
                 try{
                     scanner.claim(0);
                 }catch(JposException e){
                     JOptionPane.showMessageDialog(null, "Failed to claim \""+logicalName+"\"\nException: "+ e.getMessage(), "Failed", JOptionPane.ERROR_MESSAGE);
                     System.err.println("Jpos exception on claim " + e);
                 }
             }
             else if(ae.getActionCommand().equals("release")){
                 try{
                     scanner.release();
                 }catch(JposException e){
                     JOptionPane.showMessageDialog(null, "Failed to release \""+logicalName+"\"\nException: "+ e.getMessage(), "Failed", JOptionPane.ERROR_MESSAGE);
                     System.err.println("Jpos exception on release " + e);
                 }
             }
             else if(ae.getActionCommand().equals("close")){
                 try{
					scanner.close();
					updateDevice = false;
					claimedCB.setSelected(false);
					autoDisableCB.setSelected(false);
					deviceEnabledCB.setSelected(false);
					dataEventEnabledCB.setSelected(false);
					freezeEventsCB.setSelected(false);
					decodeDataCB.setSelected(false);
					checkHealthCB.setSelected(false);
					updateDevice = true;
					if ( scn2Open )
					{
						scanner2.close();
						scn2Open = false;
					}
                 }catch(JposException e){
                     JOptionPane.showMessageDialog(null, "Failed to close \""+logicalName+"\"\nException: "+ e.getMessage(), "Failed", JOptionPane.ERROR_MESSAGE);
                     System.err.println("Jpos exception on close" + e);
                 }
             }
             else if(ae.getActionCommand().equals("info")){
             	try{
					if ( scanner.getState() != JposConst.JPOS_S_CLOSED )
					{
             		String ver = new Integer(scanner.getDeviceServiceVersion()).toString();
             		String msg = "Service Description: " + scanner.getDeviceServiceDescription();
             		msg = msg + "\nService Version: v"+ new Integer(ver.substring(0,1)) + "." + new Integer(ver.substring(1,4)) + "." + new Integer(ver.substring(4,7));
             		ver = new Integer(scanner.getDeviceControlVersion()).toString();
             		msg += "\n\nControl Description: " + scanner.getDeviceControlDescription();
             		msg += "\nControl Version: v"+ new Integer(ver.substring(0,1)) + "." + new Integer(ver.substring(1,4)) + "." + new Integer(ver.substring(4,7));
             		msg += "\n\nPhysical Device Name: " + scanner.getPhysicalDeviceName();
             		msg += "\nPhysical Device Description: " + scanner.getPhysicalDeviceDescription();
             		
             		msg += "\n\nProperties:\n------------------------";
             		msg += "\nCapPowerReporting: " + (scanner.getCapPowerReporting() == JposConst.JPOS_PR_ADVANCED ? "Advanced" : (scanner.getCapPowerReporting() == JposConst.JPOS_PR_STANDARD ? "Standard" : "None"));
             		JOptionPane.showMessageDialog(null, msg, "Info", JOptionPane.INFORMATION_MESSAGE);
					}
             	}catch(JposException e){
             		JOptionPane.showMessageDialog(null, "Exception in Info\nException: "+ e.getMessage(), "Exception", JOptionPane.ERROR_MESSAGE);
             		System.err.println("Jpos exception " + e);
             	}
             }
             else if(ae.getActionCommand().equals("oce"))
             {
                try
                {
                    if ( logicalName.equals("") )
                    {
                        logicalName = defaultLogicalName;
                    }
                    if ( logicalName.equals("testopen") )
                    {
                        int i;
                        for ( i = 0; i < 5000; ++i )
                        {
                            System.out.println("Test cycle: " + Integer.toString(i));
                            logicalName = "SymbolScannerUSB";
                            scanner.open(logicalName);
                            scanner.claim(1000);
                            scanner.setDeviceEnabled(true);
                            scanner.setDataEventEnabled(true);
                            scanner.close();
                        }
                    }
                    else
                    {
                        if ( logicalName.equals("test2") )
                        {
                            logicalName = "SymbolUSBHandHeld";
                            try
                            {
                                scanner2.open("SymbolUSBTableTop");
                                scn2Open = true;
                                scanner2.claim(1000);
                                scanner2.setDeviceEnabled(true);
                                scanner2.setDataEventEnabled(true);
                            }
                            catch(JposException e)
                            {
                                System.out.println("Jpos exception " + e);
                            }
                        }
                        scanner.open(logicalName);
                        scanner.claim(1000);
                        scanner.setDeviceEnabled(true);
                        scanner.setDataEventEnabled(true);
                    }
                }
                catch(JposException e)
                {
                    JOptionPane.showMessageDialog(null, "Failed to claim \""+logicalName+"\"\nException: "+ e.getMessage(), "Failed", JOptionPane.ERROR_MESSAGE);
                    System.err.println("Jpos exception " + e);
                }
             }
             if(scanner.getState() != JposConst.JPOS_S_CLOSED){
	                 updateGUI();
             }
             
        }

    public static String getBarcodeTypeName(int code){
    	String val = "Unknown";
    	switch (code){
    		case ScannerConst.SCAN_SDT_UPCA:
    			val = "UPC-A";
    			break;
    		case ScannerConst.SCAN_SDT_UPCE:
    			val = " UPC-E";
    			break;
    		case ScannerConst.SCAN_SDT_JAN8:
    			val = "JAN 8 / EAN 8";
    			break;
    		case ScannerConst.SCAN_SDT_JAN13:
    			val = "JAN 13 / EAN 13";
    			break;
    		case ScannerConst.SCAN_SDT_TF:
    			val = "2 of 5";
    			break;
    		case ScannerConst.SCAN_SDT_ITF:
    			val = "Interleaved 2 of 5";
    			break;
    		case ScannerConst.SCAN_SDT_Codabar:
    			val = "Codabar";
    			break;
    		case ScannerConst.SCAN_SDT_Code39:
    			val = "Code 39";
    			break;
    		case ScannerConst.SCAN_SDT_Code93:
    			val = "Code 93";
    			break;
    		case ScannerConst.SCAN_SDT_Code128:
    			val = "Code 128";
    			break;
    		case ScannerConst.SCAN_SDT_UPCA_S:
    			val = " UPC-A with Supplemental";
    			break;
    		case ScannerConst.SCAN_SDT_UPCE_S:
    			val = "UPC-E with Supplemental";
    			break;
    		case ScannerConst.SCAN_SDT_UPCD1:
    			val = "UPC-D1";
    			break;
    		case ScannerConst.SCAN_SDT_UPCD2:
    			val = "UPC-D2";
    			break;
    		case ScannerConst.SCAN_SDT_UPCD3:
    			val = "UPC-D3";
    			break;
    		case ScannerConst.SCAN_SDT_UPCD4:
    			val = "UPC-D4";
    			break;
    		case ScannerConst.SCAN_SDT_UPCD5:
    			val = "UPC-D5";
    			break;
    		case ScannerConst.SCAN_SDT_EAN8_S:
    			val = "EAN-8 with Supplemental";
    			break;
    		case ScannerConst.SCAN_SDT_EAN13_S:
    			val = "EAN-13 with Supplemental";
    			break;
    		case ScannerConst.SCAN_SDT_EAN128:
    			val = "EAN-128";
    			break;
    		case ScannerConst.SCAN_SDT_OCRA:
    			val = "OCR \"A\"";
    			break;
    		case ScannerConst.SCAN_SDT_OCRB:
    			val = "OCR \"B\"";
    			break;
    		case ScannerConst.SCAN_SDT_PDF417:
    			val = "PDF 417";
    			break;
    		case ScannerConst.SCAN_SDT_MAXICODE:
    			val = "MAXICODE";
    			break;
    		case ScannerConst.SCAN_SDT_OTHER:
    			val = "Other";
    			break;
    	}
    	return val;
    }

    public void dataOccurred(DataEvent de)
    {
        try
        {
			Scanner scn = (Scanner) de.getSource();
			if ( scn.equals(scanner) )
			{
				if ( autoDataEventEnableCB.isSelected() )
					scanner.setDataEventEnabled(true);
				scanData = scanner.getScanData();
				scanDataLabel = scanner.getScanDataLabel();
				scanDataType = scanner.getScanDataType();
				autoDisable = scanner.getAutoDisable();
				dataEventEnabled = scanner.getDataEventEnabled();
				deviceEnabled = scanner.getDeviceEnabled();
				freezeEvents = scanner.getFreezeEvents();
				decodeData = scanner.getDecodeData();
				scanDataCount = scanner.getDataCount();
			}
			else
			{
				scn.setDataEventEnabled(true);
				scanData = scn.getScanData();
				scanDataLabel = scn.getScanDataLabel();
				scanDataType = scn.getScanDataType();
				decodeData = scn.getDecodeData();
				scanDataCount = scn.getDataCount();
			}
		}
        catch(JposException je)
        {
            System.err.println("Scanner: dataOccurred: Jpos Exception" + je);
        }

        updateDevice = false;

        try{
            SwingUtilities.invokeLater(doUpdateGUI);
        }catch(Exception e){
            System.err.println("InvokeLater exception.");
        } 
        updateDevice = true;

    }


    public void errorOccurred(ErrorEvent ee)
    {
            System.out.println("Error Occurred");
    }

    public void updateGUI()
    {
            scanDataTextField.setText(new String(scanData));
            scanDataLabelTextField.setText(new String(scanDataLabel));
            Integer i = new Integer(scanDataType);
            scanDataTypeTextField.setText(Integer.toString(scanDataType) + " ("+getBarcodeTypeName(scanDataType)+")");
			scanDataCountField.setText("0");
         //   autoDisableCB.setSelected(autoDisable);
         //   deviceEnabledCB.setSelected(deviceEnabled);
         //   dataEventEnabledCB.setSelected(dataEventEnabled);
         //   freezeEventsCB.setSelected(freezeEvents);
         //   decodeDataCB.setSelected(decodeData);
            try
            {
				if ( scanner.getState() != JposConst.JPOS_S_CLOSED )
				{
					updateDevice = false;
					autoDisableCB.setSelected(scanner.getAutoDisable());
					dataEventEnabledCB.setSelected(scanner.getDataEventEnabled());
					freezeEventsCB.setSelected(scanner.getFreezeEvents());
					decodeDataCB.setSelected(scanner.getDecodeData());
					if ( scanner.getClaimed() )
					{
						claimedCB.setSelected(true);
						deviceEnabledCB.setSelected(scanner.getDeviceEnabled());
						scanDataCount = scanner.getDataCount();
						scanDataCountField.setText(Integer.toString(scanDataCount));
					}
					else
					{
						claimedCB.setSelected(false);
						deviceEnabledCB.setSelected(false);
					}
					updateDevice = true;
				}
            }
            catch(JposException je)
            {
            	System.err.println("ScannerPanel: updateGUI method received JposException: "+ je);
            }
    }

    

    class CheckBoxListener implements ItemListener 
    {
        public void itemStateChanged(ItemEvent e) 
        {
            if(updateDevice)
            {
                Object source = e.getItemSelectable();
                if (source == autoDisableCB)
                {
                    try
                    {
                        if (e.getStateChange() == ItemEvent.DESELECTED){
                            scanner.setAutoDisable(false);
                        }else{
                            scanner.setAutoDisable(true);
                        }
                    }
                    catch(JposException je)
                    {
                        System.err.println("ScannerPanel: CheckBoxListener: Jpos Exception: " + je + "\nSource: " + source);
                    }
                }
                else if (source == dataEventEnabledCB)
                {
                    try
                    {
                        if (e.getStateChange() == ItemEvent.DESELECTED){
                            scanner.setDataEventEnabled(false);
                        }else{
                            scanner.setDataEventEnabled(true);
                        }
                    }
                    catch(JposException je)
                    {
                        System.err.println("ScannerPanel: CheckBoxListener: Jpos Exception: " + je + "\nSource: " + source);
                    }

                }
                else if (source == deviceEnabledCB)
                {
                    try
                    {
                        if (e.getStateChange() == ItemEvent.DESELECTED){
                            scanner.setDeviceEnabled(false);
                        }else{
                            scanner.setDeviceEnabled(true);
                        }
                    }
                    catch(JposException je)
                    {
                        System.err.println("ScannerPanel: CheckBoxListener: Jpos Exception: " + je + "\nSource: " + source);
                    }
                }
                else if (source == freezeEventsCB)
                {
                    try
                    {
                        if (e.getStateChange() == ItemEvent.DESELECTED){
                            scanner.setFreezeEvents(false);
                        }else{
                            scanner.setFreezeEvents(true);
                        }
                    }
                    catch(JposException je)
                    {
                        System.err.println("ScannerPanel: CheckBoxListener: Jpos Exception: " + je + "\nSource: " + source);
                    }
                }
                else if (source == decodeDataCB)
                {
                    try
                    {
                        if (e.getStateChange() == ItemEvent.DESELECTED){
                            scanner.setDecodeData(false);
                        }else{
                            scanner.setDecodeData(true);
                        }
                    }
                    catch(JposException je)
                    {
                        System.err.println("ScannerPanel: CheckBoxListener: Jpos Exception: " + je + "\nSource: " + source);
                    }
                }
				else if (source == checkHealthCB)
				{
					String healthText;
					try
					{
						checkHealthCB.setSelected(false);
						scanner.checkHealth(JposConst.JPOS_CH_INTERNAL);
						healthText = scanner.getCheckHealthText();
					}
					catch(JposException je)
					{
						healthText = je.getMessage();
					}
					scanHealthField.setText(healthText);
				}
				updateGUI();
             }


         }
     }    
    


}
